from __future__ import unicode_literals

ENVIRONMENT_TEST_NAME = 'PAGINATE_BY'
ENVIRONMENT_TEST_VALUE = '999999'

TEST_NAMESPACE_LABEL = 'test namespace label'
TEST_NAMESPACE_NAME = 'test_namespace_name'
