{% load i18n %}

{% load smart_settings_tags %}

{% if autoadmin_properties.account %}
    <div class="row">
        <div class="col-xs-10 col-xs-offset-1 col-sm-8 col-sm-offset-2 col-md-6 col-md-offset-3 col-lg-4 col-lg-offset-4">
            <br>
            <div class="panel panel-primary">
                <div class="panel-heading">
                    <h3 class="panel-title">{% trans 'First time login' %}</h3>
                </div>
                <div class="panel-body">
                    <div class="content login">
                        {% smart_setting 'COMMON_PROJECT_TITLE' as project_title %}
                        <p>{% blocktrans %}You have just finished installing <strong>{{ project_title }}</strong>, congratulations!{% endblocktrans %}</p>
                        <p>{% trans 'Login using the following credentials:' %}</p>
                        <p>{% blocktrans with autoadmin_properties.account as account %}Username: <strong>{{ account }}</strong>{% endblocktrans %}</p>
                        <p>{% blocktrans with autoadmin_properties.account.email as email %}Email: <strong>{{ email }}</strong>{% endblocktrans %}</p>
                        <p>{% blocktrans with autoadmin_properties.password as password %}Password: <strong>{{ password }}</strong>{% endblocktrans %}</p>
                        <p>{% trans 'Be sure to change the password to increase security and to disable this message.' %}</p>
                    </div>
                </div>
            </div>
        </div>
    </div>
{% endif %}
