from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon

icon_key_setup = Icon(driver_name='fontawesome', symbol='key')
icon_key_upload = Icon(driver_name='fontawesome', symbol='upload')
icon_keyserver_search = Icon(driver_name='fontawesome', symbol='search')
