from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon

icon_task_manager = Icon(driver_name='fontawesome', symbol='braille')
icon_queue_list = Icon(driver_name='fontawesome', symbol='braille')
