from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon
from mayan.apps.documents.icons import icon_document, icon_document_type


icon_workflow = Icon(driver_name='fontawesome', symbol='sitemap')

icon_document_type_workflow_list = icon_workflow

icon_document_workflow_instance_list = Icon(
    driver_name='fontawesome', symbol='sitemap'
)
icon_setup_workflow_list = Icon(driver_name='fontawesome', symbol='sitemap')
icon_tool_launch_all_workflows = Icon(
    driver_name='fontawesome', symbol='sitemap'
)
icon_workflow_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='sitemap',
    secondary_symbol='plus'
)
icon_workflow_delete = Icon(driver_name='fontawesome', symbol='times')
icon_workflow_document_type_list = icon_document_type
icon_workflow_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_workflow_list = Icon(driver_name='fontawesome', symbol='sitemap')
icon_workflow_preview = Icon(driver_name='fontawesome', symbol='eye')


icon_workflow_instance_detail = Icon(driver_name='fontawesome', symbol='sitemap')
icon_workflow_instance_transition = Icon(
    driver_name='fontawesome', symbol='arrows-alt-h'
)

icon_workflow_runtime_proxy_document_list = icon_document
icon_workflow_runtime_proxy_list = Icon(
    driver_name='fontawesome', symbol='sitemap'
)
icon_workflow_runtime_proxy_state_document_list = icon_document
icon_workflow_runtime_proxy_state_list = Icon(
    driver_name='fontawesome', symbol='circle'
)

icon_workflow_state_action_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_workflow_state_action_edit = Icon(
    driver_name='fontawesome', symbol='pencil-alt'
)
icon_workflow_state_action_list = Icon(
    driver_name='fontawesome', symbol='code'
)
icon_workflow_state = Icon(driver_name='fontawesome', symbol='circle')
icon_workflow_state_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='circle',
    secondary_symbol='plus'
)
icon_workflow_state_delete = Icon(driver_name='fontawesome', symbol='times')
icon_workflow_state_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')

icon_workflow_state_action = Icon(driver_name='fontawesome', symbol='code')
icon_workflow_state_action_delete = Icon(driver_name='fontawesome', symbol='times')
icon_workflow_state_action_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_workflow_state_action_selection = Icon(
    driver_name='fontawesome-dual', primary_symbol='code',
    secondary_symbol='plus'
)
icon_workflow_state_action_list = Icon(driver_name='fontawesome', symbol='code')
icon_workflow_transition = Icon(
    driver_name='fontawesome', symbol='arrows-alt-h'
)
icon_workflow_transition_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='arrows-alt-h',
    secondary_symbol='plus'
)
icon_workflow_transition_delete = Icon(driver_name='fontawesome', symbol='times')
icon_workflow_transition_edit = Icon(
    driver_name='fontawesome', symbol='pencil-alt'
)
icon_workflow_transition_triggers = Icon(
    driver_name='fontawesome', symbol='bolt'
)
