<span class="fa-layers fa-fw" style="margin-right: 7px;">
    {% for entry in data %}
        <i class="{{ entry.class }}" data-fa-transform="{{ entry.transform }}" data-fa-mask="{{ entry.mask }}"></i>
    {% endfor %}
</span>
